@@ -13,9 +13,11 @@ import (
 	"github.com/gohugoio/hugo/hugofs"
 	"github.com/gohugoio/hugo/langs"
 	"github.com/gohugoio/hugo/media"
+	"github.com/gohugoio/hugo/resources/page"
+
 	"github.com/gohugoio/hugo/metrics"
 	"github.com/gohugoio/hugo/output"
-	"github.com/gohugoio/hugo/resource"
+	"github.com/gohugoio/hugo/resources"
 	"github.com/gohugoio/hugo/source"
 	"github.com/gohugoio/hugo/tpl"
 	jww "github.com/spf13/jwalterweatherman"
@@ -32,6 +34,9 @@ type Deps struct {
 	// Used to log errors that may repeat itself many times.
 	DistinctErrorLog *helpers.DistinctLogger
 
+	// Used to log warnings that may repeat itself many times.
+	DistinctWarningLog *helpers.DistinctLogger
+
 	// The templates to use. This will usually implement the full tpl.TemplateHandler.
 	Tmpl tpl.TemplateFinder `json:"-"`
 
@@ -51,7 +56,7 @@ type Deps struct {
 	SourceSpec *source.SourceSpec `json:"-"`
 
 	// The Resource Spec to use
-	ResourceSpec *resource.Spec
+	ResourceSpec *resources.Spec
 
 	// The configuration to use
 	Cfg config.Provider `json:"-"`
@@ -62,8 +67,12 @@ type Deps struct {
 	// The translation func to use
 	Translate func(translationID string, args ...interface{}) string `json:"-"`
 
+	// The language in use. TODO(bep) consolidate with site
 	Language *langs.Language
 
+	// The site building.
+	Site page.Site
+
 	// All the output formats available for the current site.
 	OutputFormatsConfig output.Formats
 
@@ -118,6 +127,9 @@ type Listeners struct {
 
 // Add adds a function to a Listeners instance.
 func (b *Listeners) Add(f func()) {
+	if b == nil {
+		return
+	}
 	b.Lock()
 	defer b.Unlock()
 	b.listeners = append(b.listeners, f)
@@ -147,11 +159,11 @@ func (d *Deps) TemplateHandler() tpl.TemplateHandler {
 func (d *Deps) LoadResources() error {
 	// Note that the translations need to be loaded before the templates.
 	if err := d.translationProvider.Update(d); err != nil {
-		return err
+		return errors.Wrap(err, "loading translations")
 	}
 
 	if err := d.templateProvider.Update(d); err != nil {
-		return err
+		return errors.Wrap(err, "loading templates")
 	}
 
 	return nil
@@ -187,10 +199,18 @@ func New(cfg DepsCfg) (*Deps, error) {
 		fs = hugofs.NewDefault(cfg.Language)
 	}
 
-	ps, err := helpers.NewPathSpec(fs, cfg.Language)
+	if cfg.MediaTypes == nil {
+		cfg.MediaTypes = media.DefaultTypes
+	}
+
+	if cfg.OutputFormats == nil {
+		cfg.OutputFormats = output.DefaultFormats
+	}
+
+	ps, err := helpers.NewPathSpec(fs, cfg.Language, logger)
 
 	if err != nil {
-		return nil, err
+		return nil, errors.Wrap(err, "create PathSpec")
 	}
 
 	fileCaches, err := filecache.NewCaches(ps)
@@ -198,7 +218,7 @@ func New(cfg DepsCfg) (*Deps, error) {
 		return nil, errors.WithMessage(err, "failed to create file caches from configuration")
 	}
 
-	resourceSpec, err := resource.NewSpec(ps, fileCaches, logger, cfg.OutputFormats, cfg.MediaTypes)
+	resourceSpec, err := resources.NewSpec(ps, fileCaches, logger, cfg.OutputFormats, cfg.MediaTypes)
 	if err != nil {
 		return nil, err
 	}
@@ -216,11 +236,13 @@ func New(cfg DepsCfg) (*Deps, error) {
 	}
 
 	distinctErrorLogger := helpers.NewDistinctLogger(logger.ERROR)
+	distinctWarnLogger := helpers.NewDistinctLogger(logger.WARN)
 
 	d := &Deps{
 		Fs:                  fs,
 		Log:                 logger,
 		DistinctErrorLog:    distinctErrorLogger,
+		DistinctWarningLog:  distinctWarnLogger,
 		templateProvider:    cfg.TemplateProvider,
 		translationProvider: cfg.TranslationProvider,
 		WithTemplate:        cfg.WithTemplate,
@@ -230,6 +252,7 @@ func New(cfg DepsCfg) (*Deps, error) {
 		ResourceSpec:        resourceSpec,
 		Cfg:                 cfg.Language,
 		Language:            cfg.Language,
+		Site:                cfg.Site,
 		FileCaches:          fileCaches,
 		BuildStartListeners: &Listeners{},
 		Timeout:             time.Duration(timeoutms) * time.Millisecond,
@@ -245,11 +268,11 @@ func New(cfg DepsCfg) (*Deps, error) {
 
 // ForLanguage creates a copy of the Deps with the language dependent
 // parts switched out.
-func (d Deps) ForLanguage(cfg DepsCfg) (*Deps, error) {
+func (d Deps) ForLanguage(cfg DepsCfg, onCreated func(d *Deps) error) (*Deps, error) {
 	l := cfg.Language
 	var err error
 
-	d.PathSpec, err = helpers.NewPathSpecWithBaseBaseFsProvided(d.Fs, l, d.BaseFs)
+	d.PathSpec, err = helpers.NewPathSpecWithBaseBaseFsProvided(d.Fs, l, d.Log, d.BaseFs)
 	if err != nil {
 		return nil, err
 	}
@@ -259,10 +282,12 @@ func (d Deps) ForLanguage(cfg DepsCfg) (*Deps, error) {
 		return nil, err
 	}
 
+	d.Site = cfg.Site
+
 	// The resource cache is global so reuse.
 	// TODO(bep) clean up these inits.
 	resourceCache := d.ResourceSpec.ResourceCache
-	d.ResourceSpec, err = resource.NewSpec(d.PathSpec, d.ResourceSpec.FileCaches, d.Log, cfg.OutputFormats, cfg.MediaTypes)
+	d.ResourceSpec, err = resources.NewSpec(d.PathSpec, d.ResourceSpec.FileCaches, d.Log, cfg.OutputFormats, cfg.MediaTypes)
 	if err != nil {
 		return nil, err
 	}
@@ -271,6 +296,12 @@ func (d Deps) ForLanguage(cfg DepsCfg) (*Deps, error) {
 	d.Cfg = l
 	d.Language = l
 
+	if onCreated != nil {
+		if err = onCreated(&d); err != nil {
+			return nil, err
+		}
+	}
+
 	if err := d.translationProvider.Clone(&d); err != nil {
 		return nil, err
 	}
@@ -299,6 +330,9 @@ type DepsCfg struct {
 	// The language to use.
 	Language *langs.Language
 
+	// The Site in use
+	Site page.Site
+
 	// The configuration to use.
 	Cfg config.Provider
 
