@@ -1,4 +1,4 @@
-// Copyright 2017-present The Hugo Authors. All rights reserved.
+// Copyright 2019 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -15,12 +15,10 @@ package hugolib
 
 import (
 	"fmt"
-	"path"
-	"strings"
 
 	"github.com/gohugoio/hugo/config"
-	"github.com/gohugoio/hugo/helpers"
 	"github.com/gohugoio/hugo/output"
+	"github.com/gohugoio/hugo/resources/page"
 	"github.com/spf13/cast"
 )
 
@@ -30,31 +28,17 @@ func createDefaultOutputFormats(allFormats output.Formats, cfg config.Provider)
 	robotsOut, _ := allFormats.GetByName(output.RobotsTxtFormat.Name)
 	sitemapOut, _ := allFormats.GetByName(output.SitemapFormat.Name)
 
-	// TODO(bep) this mumbo jumbo is deprecated and should be removed, but there are tests that
-	// depends on this, so that will have to wait.
-	rssBase := cfg.GetString("rssURI")
-	if rssBase == "" || rssBase == "index.xml" {
-		rssBase = rssOut.BaseName
-	} else {
-		// Remove in Hugo 0.36.
-		helpers.Deprecated("Site config", "rssURI", "Set baseName in outputFormats.RSS", true)
-		// RSS has now a well defined media type, so strip any suffix provided
-		rssBase = strings.TrimSuffix(rssBase, path.Ext(rssBase))
-	}
-
-	rssOut.BaseName = rssBase
-
 	return map[string]output.Formats{
-		KindPage:         output.Formats{htmlOut},
-		KindHome:         output.Formats{htmlOut, rssOut},
-		KindSection:      output.Formats{htmlOut, rssOut},
-		KindTaxonomy:     output.Formats{htmlOut, rssOut},
-		KindTaxonomyTerm: output.Formats{htmlOut, rssOut},
-		// Below are for conistency. They are currently not used during rendering.
-		kindRSS:       output.Formats{rssOut},
-		kindSitemap:   output.Formats{sitemapOut},
-		kindRobotsTXT: output.Formats{robotsOut},
-		kind404:       output.Formats{htmlOut},
+		page.KindPage:         {htmlOut},
+		page.KindHome:         {htmlOut, rssOut},
+		page.KindSection:      {htmlOut, rssOut},
+		page.KindTaxonomy:     {htmlOut, rssOut},
+		page.KindTaxonomyTerm: {htmlOut, rssOut},
+		// Below are for consistency. They are currently not used during rendering.
+		kindRSS:       {rssOut},
+		kindSitemap:   {sitemapOut},
+		kindRobotsTXT: {robotsOut},
+		kind404:       {htmlOut},
 	}
 
 }
@@ -70,19 +54,24 @@ func createSiteOutputFormats(allFormats output.Formats, cfg config.Provider) (ma
 
 	outputs := cfg.GetStringMap("outputs")
 
-	if outputs == nil || len(outputs) == 0 {
+	if len(outputs) == 0 {
 		return outFormats, nil
 	}
 
 	seen := make(map[string]bool)
 
 	for k, v := range outputs {
+		k = getKind(k)
+		if k == "" {
+			// Invalid kind
+			continue
+		}
 		var formats output.Formats
 		vals := cast.ToStringSlice(v)
 		for _, format := range vals {
 			f, found := allFormats.GetByName(format)
 			if !found {
-				return nil, fmt.Errorf("Failed to resolve output format %q from site config", format)
+				return nil, fmt.Errorf("failed to resolve output format %q from site config", format)
 			}
 			formats = append(formats, f)
 		}
